<?php
/**
 * TMFColorParser v1.3c by oorf|fuckfish (fish@stabb.de)
 * Modified by CavalierDeVache to drop into Cyphers TM Aseco stats,
 * added additional UTF-8 and Pan-Unicode font support.
 */
class ColorParser{

	var $links, $manialinks, $background;
	var $forceDarken, $forceBrighten;
	var $alwaysDrawFontShadows;

	function ColorParser($autoContrastToBackgroundColor = ''){
		$this->autoContrastColor($autoContrastToBackgroundColor);
		$this->forceDarken = false;
		$this->forceBrighten = false;
		$this->alwaysDrawFontShadows = true;
	}

	function getStyledString($string, $match, $col, $wide, $narrow, $caps, $italic, $stripColors){

		$string = mb_substr($string, mb_strlen($match));

		if ($caps) $string = mb_strtoupper($string);

		if (($col||$wide||$narrow||$italic)&&($string)){
			$start1 = "<span style='";
			$start2 = "'>";
			$styles = "";
			$end = "</span>";
			if ($col && !$stripColors){
				$colRGB = $this->get_rgb('#'.$col);
				$colRGBNew = $this->getContrastCorrectedColor($colRGB);
				$colNew = $this->get_hex($colRGBNew);
				$styles.="color:".$colNew.";";
			}
			if ($italic){
				$styles.="font-style:italic;";
			}
			if ($wide){
				$styles.="font-weight:bold;";
			}
			if ($narrow){
				$styles.="letter-spacing: -0.1em;font-size:smaller";
			}
			$string = $start1.$styles.$start2.$string.$end;
		}
		return $string;
	}

	function parseLinks($str, $showLinks = true){
		$str = $this->parseManialinks($str, $showLinks);
		$str = str_replace('$L', '$l', $str);
		$this->links=array();
		$linkidx = 0;
		$chunks = explode('$l', $str);
		for ($i=0; $i<count($chunks); $i++){
			$text = $chunks[$i];
			if ($i%2==1){
				$id = '{LINK'.$linkidx.'}';
				$linkidx++;
				if (preg_match('/\A\[(.*)\](.*)\Z/u', $text)) {
					$url = mb_substr($text, mb_strpos($text, '[')+1);
					$url = mb_substr($url, 0, mb_strpos($url, ']'));
					$this->links[$id] = $url;
					if ($showLinks) $chunks[$i] = '$a'.$id.'$x'.mb_substr($text, mb_strpos($text, ']')+1).'$a{/LINK}$x';
					else $chunks[$i] = mb_substr($text, mb_strpos($text, ']')+1);
				} else {
					$this->links[$id]= $text;
					if ($showLinks) $chunks[$i] = '$a'.$id.'$x'.$text.'$a{/LINK}$x';
				}
			}
		}
		$this->fixWWWLinks();
		return implode('', $chunks);
	}
	
	function linkIsIP($link){
		return (preg_match('/\\A\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}.*\\z/u', $link));
	}

	function fixWWWLinks(){
		foreach ($this->links as $key => $value){
			$value = trim($value);
			if (mb_substr(mb_strtolower($value), 0, 4) == 'www.'){
				$value = 'http://'.$value;
				$this->links[$key] = $value;
			} else if ($this->linkIsIP($value)){
				$value = 'http://'.$value;
				$this->links[$key] = $value;
			}
		}
	}

	function parseManialinks($str, $showLinks = true) {
		$str = str_replace('$H', '$h', $str);
		$this->manialinks=array();
		$linkidx = 0;
		$chunks = explode('$h', $str);
		for ($i=0; $i<count($chunks); $i++){
			$text = $chunks[$i];
			if ($i%2==1){
				$id = '{MLINK'.$linkidx.'}';
				$linkidx++;
				if (preg_match('/\A\[(.*)\](.*)\Z/u', $text)) {
					$url = mb_substr($text, mb_strpos($text, '[')+1);
					$url = mb_substr($url, 0, mb_strpos($url, ']'));
					$this->manialinks[$id] = $url;
					if ($showLinks) $chunks[$i] = '$a'.$id.'$x'.mb_substr($text, mb_strpos($text, ']')+1).'$a{/LINK}$x';
					else $chunks[$i] = mb_substr($text, mb_strpos($text, ']')+1);
				} else {
					$this->manialinks[$id]= $text;
					if ($showLinks) $chunks[$i] = '$a'.$id.'$x'.$text.'$a{/LINK}$x';
				}
			}
		}
		return implode('', $chunks);
	}


	function insertLinks($str){
		foreach ($this->manialinks as $key => $value){
			$str = str_replace($key,  '<a href ="tmtp:///:'.$value.'">', $str);
		}
		foreach ($this->links as $key => $value){
			$str = str_replace($key, '<a href="'.$value.'">', $str);
		}
		$str = str_replace('{/LINK}', '</a>', $str);
		return $str;
	}


	function getColorBrightness($rgb){
		return ($rgb['r']*299 + $rgb['g']*587  + $rgb['b']*114) / 1000;
	}

	function getBrightnessDifference($rgb1, $rgb2){
		return (abs($this->getColorBrightness($rgb1) - $this->getColorBrightness($rgb2)));
	}

	function getColorDifference($rgb1, $rgb2){
		$rDiff = abs($rgb1['r'] - $rgb2['r']);
		$gDiff = abs($rgb1['g'] - $rgb2['g']);
		$bDiff = abs($rgb1['b'] - $rgb2['b']);
		$rgbDiff = ($rDiff + $gDiff + $bDiff) / 768 * 100;
		return $rgbDiff;
	}

	function autoContrastColor($bgColor){
		$this->background = $this->get_rgb($bgColor);
	}

	function getContrastCorrectedColor($rgb){
		if (!$this->background) return $rgb;
		$lighter = $rgb;
		$darker = $rgb;
		$diff = $this->getColorDifference($rgb, $this->background);
		$white = $this->get_rgb('#ffffff');
		$black = $this->get_rgb('#000000');

		$limit = 15;
		$steps = 50;
		$diffDarkerFromRealColor = 255;
		$diffLighterFromRealColor = 255;

		for ($i = 1; $i<=$steps; $i++){
			$diffLight = $this->getColorDifference($lighter, $this->background);
			$diffDark = $this->getColorDifference($darker, $this->background);
			if ($diffLight < $limit){
				$lighter['r'] = ($steps-$i)/$steps * $rgb['r'] + $i/$steps * $white['r'];
				$lighter['g'] = ($steps-$i)/$steps * $rgb['g'] + $i/$steps * $white['g'];
				$lighter['b'] = ($steps-$i)/$steps * $rgb['b'] + $i/$steps * $white['b'];
			} else {
				$diffLighterFromRealColor = $this->getColorDifference($lighter, $rgb);
			}
			if ($diffDark < $limit){
				$darker['r'] = ($steps-$i)/$steps * $rgb['r'] + $i/$steps * $black['r'];
				$darker['g'] = ($steps-$i)/$steps * $rgb['g'] + $i/$steps * $black['g'];
				$darker['b'] = ($steps-$i)/$steps * $rgb['b'] + $i/$steps * $black['b'];
			} else {
				$diffDarkerFromRealColor = $this->getColorDifference($darker, $rgb);
			}
		}
		$totalDiffLight = abs($limit - $this->getColorDifference($lighter, $this->background));
		$totalDiffDark = abs($limit - $this->getColorDifference($darker, $this->background));


		if ($this->forceDarken) return $darker;
		if ($this->forceBrighten) return $lighter;
		if ($diffLighterFromRealColor < $diffDarkerFromRealColor){
			return $lighter;
		} else if ($diffLighterFromRealColor > $diffDarkerFromRealColor){
			return $darker;
		} else {
			if ($totalDiffDark < $totalDiffLight) return $darker; else return $lighter;
		}

	}

	function toHTML($str, $stripColors = false, $stripLinks = false, $stripTags = ''){
		$col = false;
		$wide = false;
		$narrow = false;
		$caps = false;
		$italic = false;

		$str = $this->getValidUTF8String($str);
		$str = str_ireplace('$foo', '$f00', $str);

		if (mb_strtolower($stripTags) == 'all') $stripTags = 'iwonstmgaxz';
		for ($i=0; $i<mb_strlen($stripTags); $i++){
			$toStrip = mb_substr($stripTags, $i, 1);
			$str = str_ireplace('$'.$toStrip, '', $str);
		}

		$str = str_replace('$$','[DOLLAR]',$str);
		$str = str_replace(' ','&nbsp;',$str);
		$str = $this->parseLinks($str, !$stripLinks);
		$chunks = explode('$',$str);

		for ($i = 1; $i < count($chunks); $i++){
			if (preg_match("/^[0-9a-f]{2,3}/iu",$chunks[$i], $matches)){
				$col = $matches[0];
				if (mb_strlen($col)<3) $col.="8";
				$col = $col[0].$col[0].$col[1].$col[1].$col[2].$col[2];
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(i)/iu",$chunks[$i], $matches)){
				$italic = true;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(w)/iu",$chunks[$i], $matches)){
				$narrow = false;
				$wide = true;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(o)/iu",$chunks[$i], $matches)){
				$narrow = false;
				$wide = true;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(n)/iu",$chunks[$i], $matches)){
				$wide = false;
				$narrow = true;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(s)/iu",$chunks[$i], $matches)){
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(t)/iu",$chunks[$i], $matches)){
				$caps = true;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			} else if (preg_match("/^(m)/iu",$chunks[$i], $matches)){
				$wide = false;
				$bold = false;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			}  else if (preg_match("/^(g)/iu",$chunks[$i], $matches)){
				$col = false;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			}  else if (preg_match("/^(a)/iu",$chunks[$i], $matches)){

				$colSave= $col;
				$wideSave = $wide;
				$narrowSave = $narrow;
				$capsSave = $caps;
				$italicSave = $italic;
				$col = false;
				$wide = false;
				$narrow = false;
				$caps = false;
				$italic = false;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			}  else if (preg_match("/^(x)/iu",$chunks[$i], $matches)){

				$col = $colSave;
				$wide = $wideSave;
				$narrow = $narrowSave;
				$caps = $capsSave;
				$italic = $italicSave;

				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			}  else if (preg_match("/^(z)/iu",$chunks[$i], $matches)){
				$col = false;
				$wide = false;
				$narrow = false;
				$caps = false;
				$italic = false;
				$chunks[$i] = $this->getStyledString($chunks[$i], $matches[0], $col, $wide, $narrow, $caps, $italic, $stripColors);
			}

		}

		for ($i = 1; $i < count($chunks); $i++){
			$chunks[$i] = str_replace('[DOLLAR]','$',$chunks[$i]);
			$chunks[$i] = str_replace('&NBSP;','&nbsp;',$chunks[$i]);

		}


		$str = implode($chunks);
		$str = $this->insertLinks($str);
		return $str;
	}
	function toArray($str){
		$col = false;
		$wide = false;
		$narrow = false;
		$caps = false;
		$italic = false;
		$shadow = false;

		$str = str_replace('$$','[DOLLAR]',$str);
		$str = $this->parseLinks($str, false);
		$chunks = explode('$',$str);

		$result = array();
		if ($chunks[0]) $result[0]["text"]=str_replace('[DOLLAR]','$',$chunks[0]);


		for ($i = 1; $i < count($chunks); $i++){
			$match = "";
			if (preg_match("/^[0-9a-f]{2,3}/iu",$chunks[$i], $matches)){
				$col = $matches[0];
				$match = $col;
				if (mb_strlen($col)<3) $col.="8";
				$col = $col[0].$col[0].$col[1].$col[1].$col[2].$col[2];
			} else if (preg_match("/^(i)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$italic = true;
			} else if (preg_match("/^(w)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$narrow = false;
				$wide = true;
			} else if (preg_match("/^(o)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$narrow = false;
				$wide = true;
			} else if (preg_match("/^(n)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$wide = false;
				$narrow = true;
			} else if (preg_match("/^(s)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$shadow = true;
			} else if (preg_match("/^(t)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$caps = true;
			} else if (preg_match("/^(m)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$wide = false;
				$bold = false;
			}  else if (preg_match("/^(g)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$col = false;
			}  else if (preg_match("/^(x)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
			}  else if (preg_match("/^(z)/iu",$chunks[$i], $matches)){
				$match = $matches[0];
				$shadow = false;
				$col = false;
				$wide = false;
				$narrow = false;
				$caps = false;
				$italic = false;
			}

			$chunks[$i] = mb_substr($chunks[$i], mb_strlen($match));
			if ($chunks[$i]){
				$a = count($result);
				$result[$a]["text"]=str_replace('[DOLLAR]','$',$chunks[$i]);
				$result[$a]["italic"]=$italic;
				$result[$a]["narrow"]=$narrow;
				$result[$a]["wide"]=$wide;
				$result[$a]["caps"]=$caps;
				$result[$a]["shadow"]=$shadow;
				$result[$a]["color"]=$col;
			}

		}

		return $result;
	}

	function get_rgb($hex) {
		$hex_array = array(0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
		'A' => 10, 'B' => 11, 'C' => 12, 'D' => 13, 'E' => 14,
		'F' => 15);
		$hex = str_replace('#', '', mb_strtoupper($hex));
		if (($length = mb_strlen($hex)) == 3) {
			$hex = $hex{0}.$hex{0}.$hex{1}.$hex{1}.$hex{2}.$hex{2};
			$length = 6;
		}
		if ($length != 6 or mb_strlen(str_replace(array_keys($hex_array), '', $hex)))
		return NULL;
		$rgb['r'] = $hex_array[$hex{0}] * 16 + $hex_array[$hex{1}];
		$rgb['g'] = $hex_array[$hex{2}] * 16 + $hex_array[$hex{3}];
		$rgb['b']= $hex_array[$hex{4}] * 16 + $hex_array[$hex{5}];
		return $rgb;
	}

	function get_hex($rgb) {
		$r = intval($rgb['r']);
		$g = intval($rgb['g']);
		$b = intval($rgb['b']);
		$r = dechex($r<0?0:($r>255?255:$r));
		$g = dechex($g<0?0:($g>255?255:$g));
		$b = dechex($b<0?0:($b>255?255:$b));
		$color = (mb_strlen($r) < 2?'0':'').$r;
		$color .= (mb_strlen($g) < 2?'0':'').$g;
		$color .= (mb_strlen($b) < 2?'0':'').$b;
		return '#'.$color;
	}

	/**
     * draws a TMN color coded string onto the given picture
     *
     * @param imageHandle $src_img The picture to be drawn on
     * @param int $size	The fontsize
     * @param int $x The X-Position
     * @param int $y The Y-Position
     * @param color $color The color for letters that are not otherwise color coded (default color)
     * @param String $font The used font
     * 					   Attention: You need 4 fonts for the different styles like
     * 								tahoma.ttf		-- normal style
     * 								tahomait.ttf	-- italic style
     * 								tahomawd.ttf    -- wide style
     * 								tahomawdit.ttf	-- wide and italic style
     * 					   (narrow style not implemented yet)
     * @param String $text The TMN color coded Text you want to display
     * @param boolean $stripColors Strip out the colors or not.
     * @param String $stripTags
     * Modified to use a single full Pan-Unicode font like Arial MS Unicode or Cyberbit if available -CavalierDeVache
     */

  	function filePath($filePath)
	{
 	$fileParts = pathinfo($filePath);

 	if(!isset($fileParts['filename']))
 	{$fileParts['filename'] = mb_substr($fileParts['basename'], 0, mb_strrpos($fileParts['basename'], '.'));}
 
 	return $fileParts;
	}

	function drawStyledString($src_img,$size, $x, $y , $color, $font, $text, $stripColors = false, $stripTags = ''){
		
		$text = $this->getValidUTF8String($text);

		if (mb_strtolower($stripTags) == 'all') $stripTags = 'iwonstmgaxz';
		for ($i=0; $i<mb_strlen($stripTags); $i++){
			$toStrip = mb_substr($stripTags, $i, 1);
			$text = str_ireplace('$'.$toStrip, '', $text);
		}

		$chunks = $this->toArray($text);
		//var_dump($text);
		$x_offset = 0;
		$black = imagecolorallocate($src_img, 50,50,50);

		for ($i = 0; $i < count ($chunks); $i++){


			$path_parts = $this->filePath($font);
			$fontUsed1 = $path_parts['filename'];
			if ($fontused1 == "tahoma") {
			if ($chunks[$i]["wide"]) { $fontUsed1 = $fontUsed1."wd"; }
			if ($chunks[$i]["italic"]) { $fontUsed1 = $fontUsed1."it"; }
			if ($chunks[$i]["caps"]) { $chunks[$i]["text"] = mb_strtoupper($chunks[$i]["text"]); }
			$fontUsed = $path_parts['dirname']."/".$fontUsed1.".ttf";
			} else {
			if ($chunks[$i]["caps"]) { $chunks[$i]["text"] = mb_strtoupper($chunks[$i]["text"]); }
			$fontUsed = $path_parts['dirname']."/".$fontUsed1.".ttf";
			$fontused = trim($fontused);
			}

			if ($chunks[$i]["color"] && (!$stripColors)){
				$colRGB = $this->get_rgb("#".$chunks[$i]["color"]);
				$colRGB = $this->getContrastCorrectedColor($colRGB);
				$usedCol = imagecolorallocate($src_img, $colRGB['r'], $colRGB['g'], $colRGB['b']);
			} else {
				$usedCol = $color;
			}

			if ($this->alwaysDrawFontShadows || $chunks[$i]['shadow'])
			imagettftext($src_img, $size, 0, $x + $x_offset + 1, $y + 1, $black, $fontUsed, $chunks[$i]["text"]);
			imagettftext($src_img, $size, 0, $x + $x_offset, $y, $usedCol, $fontUsed, $chunks[$i]["text"]);

			$bbox = imagettfbbox($size, 0, $fontUsed, $chunks[$i]["text"]);
			$x_offset += $bbox[2]+2;

		}
	}

	function forceDarkerColors(){
		$this->forceDarken = true;
		$this->forceBrighten = false;
	}

	function forceBrighterColors(){
		$this->forceDarken = false;
		$this->forceBrighten = true;
	}
	
	function forceAutomaticColorCorrection(){
		$this->forceDarken = false;
		$this->forceBrighten = false;
	}
   /* Fixes the encoding to uf8 - found on php.net */
	function fixEncoding($in_str)
		{
  		$cur_encoding = mb_detect_encoding($in_str) ;
  		if($cur_encoding == "UTF-8")
    	return $in_str;
  		else
    	return iconv($cur_encoding, "UTF-8//TRANSLIT//IGNORE", $in_str);
		} // fixEncoding 

	function getValidUTF8String($in)
	{
	$in = $this->fixEncoding($in);
	$controls = array('\r\n', '\n', '\r', '\t');
	$in = str_replace($controls, ' ',trim($in, "\xEF\xBB\xBF\x0..\x1F"));
	$out = "";
	$length = mb_strlen($in, '8bit');
		for ( $i = 0; $i < $length; $i++)
		{
		$current = ord($in{$i});

			if ( 
			($current == 0x9) || 
			($current == 0xA) || 
			($current == 0xD) || 
			(($current >= 0x20) && ($current <= 0xD7FF)) || 
			(($current >= 0xE000) && ($current <= 0xFFFD)) || 
			(($current >= 0x10000) && ($current <= 0x10FFFF))
			)
		{

        	$out .= chr($current);
		}
		else
		{
		$out .= " ";
		}
		}
	$out = iconv("UTF-8", "UTF-8//IGNORE", $out);
	return $out;
	}

}
?>
